Attribute VB_Name = "BINRES"
'
' BINRES - Library of Visual Basic 5.0 routines for handling
'          Binary Resources of any format and almost any length
'          in a disk file.
'
' "vb50lib\binres33.bas"  Global declarations and Code module
'
' Copyright IBM Global Services Australia 2001
'
' Version   Date         Author      Changes
'     3.1   08 Sep 1999  Glenn       Initial version, based on VB3.0 BINRES 2.1,
'                        Baddeley    In BinResOpen move clear or BinResError from
'                        <glenbadd@au1.ibm.com>   after OpenExit to before OpenExit
'       2   08 Sep 1999  GB          In BinResOpen with index size specified, overwrite
'                                    existing index rather than just appending a new one
'       3   19 Aug 2001  GB          Complete VB 5.0 migration
'
'
' Binary Resource file format
' ===========================
'
' File contains a long integer at offset x38 (VB seek address d57)
' which is an offset in the file to the first index block.
' If this is 0 there are no resources.
'
' Index Block format:
' 2   bytes   int         Maximum length of index entries
' 2   bytes   int         Used length of index entries (n)
' n   bytes   bin         Concatenated index entries
'
' Index Entry format:
' 4   bytes   long int    Offset to resource binary data
' 4   bytes   long int    Length of resource binary data
' 1   byte    short int   Length of resource identifier (m)
' m   bytes   ASCII       Resource identifier
'
' Resource binary data is concatenated on byte boundaries,
' commencing after the index block.
'

' Library Routines
' =================
'
' BinResOpen( ResFileName, InitIndexLength )
'   Opens Binary Resource file for access.
'   InitIndexLength is the maximum length of index entries to initialise
'   a resource and allow BinResAppend and BinResAppendFile to be used.
'   A zero length indicates read only access to the resource.
'
' BinResClose( ResFileName )
'   Closes Binary Resource file from access.
'
' ResourceBuffer = BinResGet( ResourceId )
'   Get a resource as a string.
'
' BinResGetFile( ResourceId, FilePath )
'   Get a resource as a file.
'
' ResourceLength = BinResGetLength( ResourceId )
'   Get the length of a resource as a long integer.
'
' ResourceIdList = BinRegGetIds( )
'   Get a list of resouce Ids, separated by NUL
'
' BinResAppend( ResourceId, ResourceBuffer)
'   Append a new resource from a string.
'
' BinResAppendFile( ResourceId, FilePath )
'   Append a new resource from a file.
'
' BinResExpandFile( SoureFilePath, DestinationFilePath )
'   Read a compressed source file and expand it to a destination file.
'

' Global data
' ===========
'
' BinResError
'   Integer has value of 0 (=False) if previous BinRes routine succeeded,
'   otherwise it contains a non-zero error code.
'
' BinResStatus
'   String has value of multi line error message for last failure.
'
' BR_Fid
'   Binary resource file identifier.  Non zero if open.
'
' BR_IdxMaxEntsLen
'   Maximum length of index entries.
'
' BR_IdxUsedEntsLen
'   Used length of index entries.
'

Option Explicit

' Global

Global BinResError As Integer
Global BinResStatus As String

' Local

Const BR_BaseOff = 57           ' VB Offset in file

Const BR_MINBASE = 65           ' Minimum base for index block
Const BR_MAXDATALEN = 10000000  ' Maximum length of data (sanity check only)
Const BR_MINIDLEN = 1           ' Minimum length of identifier (sanity check only)
Const BR_MAXIDLEN = 255         ' Maximum length of identifier (sanity check only)

Const BR_BUFFSIZE = 16384

Global BR_Fid As Integer           ' Binary resource file identifier
Global BR_IdxMaxEntsLen As Integer ' Maximum length of index entries
Global BR_IdxUsedEntsLen As Integer ' Used length of index entries

Dim BR_Base As Long             ' Offset of index block
Dim BR_IdxEntries As String     ' Index entries
Dim BR_IdxUpdated As Integer    ' Flag to indicated that index block has been updated
Dim BR_Free As Long             ' Offset of next free data block

Const BR_LOOKUPBEGID = "["
Const BR_LOOKUPENDID = "]"
Dim BR_LookupId As String       ' Identifier cache for lookup, format is many of:
'                                 2 bytes  int    Byte number in index entries of entry for identifier
'                                 n bytes  ASCII  Identifier, delimited by BR_LOOKUPBEGID and BR_LOOKUPENDID

' Windows API definitions to use LZEXPAND.DLL

Type OFSTRUCT
    cBytes As String * 1
    fFixedDisk As String * 1
    nErrCode As Integer
    Reserved As String * 4
    szPathName As String * 128
End Type

Global Const OF_READ = &H0
Global Const OF_CREATE = &H1000

Declare Function LZCopy Lib "LZEXPAND.DLL" (ByVal hfSource As Integer, ByVal hfDest As Integer) As Long
Declare Function LZOpenFile Lib "LZEXPAND.DLL" (ByVal lpszFile As String, lpOf As OFSTRUCT, ByVal style As Integer) As Integer
Declare Sub LZClose Lib "LZEXPAND.DLL" (ByVal hfFile As Integer)

Sub BinResAppend(BAId As String, BAData As String)

Const Ident = "BinResAppend: "
Dim IdLen As Integer, DatLen As Long

    IdLen = Len(BAId)
    If IdLen < BR_MINIDLEN Or IdLen > BR_MAXIDLEN Then
         BinResError = 32005
         BinResStatus = Ident & "Invalid Identifier length " & IdLen
         Exit Sub
    End If
    If Len(BR_IdxEntries) + 9 + IdLen > BR_IdxMaxEntsLen Then
         BinResError = 32006
         BinResStatus = Ident & "Index is full, IdxEntries=" & Len(BR_IdxEntries) & ", IdLen=" & IdLen & ", MaxEntsLen=" & BR_IdxMaxEntsLen
         Exit Sub
    End If

    ' Add index entry
    DatLen = Len(BAData)
    If DatLen = 0 Then
        BR_IdxEntries = BR_IdxEntries & BinResLongStr(0) & BinResLongStr(0) & Chr(IdLen) & BAId
    Else
        BR_IdxEntries = BR_IdxEntries & BinResLongStr(BR_Free) & BinResLongStr(DatLen) & Chr(IdLen) & BAId
        ' Add resource binary data
        Put #BR_Fid, BR_Free, BAData
        BR_Free = BR_Free + DatLen
    End If

    ' Add Id lookup entry
    BR_LookupId = BR_LookupId & BinResIntStr(BR_IdxUsedEntsLen + 1) & BR_LOOKUPBEGID & BAId & BR_LOOKUPENDID

    BR_IdxUsedEntsLen = Len(BR_IdxEntries)
    BR_IdxUpdated = True
    BinResError = 0
    BinResStatus = ""

End Sub

Sub BinResAppendFile(BAId As String, BAFile As String)

Const Ident = "BinResAppendFile: "
Dim IdLen As Integer, FFid As Integer
Dim DatLen As Long, BOff As Long, BuffLen As Long
Dim BData As String

    On Error GoTo AppendFileError

    DatLen = FileLen(BAFile)
    If DatLen > BR_MAXDATALEN Then
        BinResError = 32004
        BinResStatus = Ident & "File length " & DatLen & " too long"
        Exit Sub
    End If

    IdLen = Len(BAId)
    If IdLen < BR_MINIDLEN Or IdLen > BR_MAXIDLEN Then
         BinResError = 32005
         BinResStatus = Ident & "Invalid Identifier length " & IdLen
         Exit Sub
    End If
    If Len(BR_IdxEntries) + 9 + IdLen > BR_IdxMaxEntsLen Then
         BinResError = 32006
         BinResStatus = Ident & "Index is full"
         Exit Sub
    End If
    
    ' Add index entry
    If DatLen = 0 Then
        BR_IdxEntries = BR_IdxEntries & BinResLongStr(0) & BinResLongStr(0) & Chr(IdLen) & BAId
    Else
        BR_IdxEntries = BR_IdxEntries & BinResLongStr(BR_Free) & BinResLongStr(DatLen) & Chr(IdLen) & BAId
        ' Add resource binary data
        FFid = FreeFile
        Open BAFile For Binary Access Read As #FFid
        BOff = 1
        BuffLen = BR_BUFFSIZE
        BData = Space(BR_BUFFSIZE)
        Do Until BOff > DatLen
            If DatLen - BOff < BR_BUFFSIZE Then
                BuffLen = DatLen - BOff + 1
                BData = Space(BuffLen)
            End If
            Get #FFid, BOff, BData
            BOff = BOff + BR_BUFFSIZE
            Put #BR_Fid, BR_Free, BData
            BR_Free = BR_Free + BuffLen
        Loop
        Close #FFid
    End If

    ' Add Id lookup entry
    BR_LookupId = BR_LookupId & BinResIntStr(BR_IdxUsedEntsLen & 1) & BR_LOOKUPBEGID & BAId & BR_LOOKUPENDID

    BR_IdxUsedEntsLen = Len(BR_IdxEntries)
    BR_IdxUpdated = True
    BinResError = 0
    BinResStatus = ""

AppendFileExit:

Exit Sub

AppendFileError:

    BinResError = Err
    BinResStatus = Ident & "Error " & Err & ": " & Error

Resume AppendFileExit

End Sub

Sub BinResClose()

Const Ident = "BinResClose: "
Dim EofMark As Integer

    On Error GoTo CloseError
    
    If BR_IdxUpdated Then
        Put #BR_Fid, BR_Base + 2, BR_IdxUsedEntsLen
        Put #BR_Fid, BR_Base + 4, BR_IdxEntries
    End If
    
    Close #BR_Fid
    BinResError = 0
    BinResStatus = ""

CloseExit:
    
    BR_Fid = 0
    BR_IdxEntries = ""
    BR_LookupId = ""

Exit Sub

CloseError:

    BinResError = Err
    BinResStatus = Ident & "Error " & Err & ": " & Error

Resume CloseExit
    
End Sub

Sub BinResExpandFile(SrcFile As String, DestFile As String)

Const Ident = "BinResExpandFile: "
Dim hfSrcFile As Integer, hfDestFile As Integer
Dim lDestLen As Long
Dim SrcOpenInfo As OFSTRUCT, DestOpenInfo As OFSTRUCT

    On Error GoTo ExpandError
    
    hfSrcFile = LZOpenFile(SrcFile, SrcOpenInfo, OF_READ)
    If hfSrcFile = -1 Then
        BinResError = 20000 + SrcOpenInfo.nErrCode
        BinResStatus = Ident & "Error opening source file"
        Exit Sub
    End If

    hfDestFile = LZOpenFile(DestFile, DestOpenInfo, OF_CREATE)
    If hfDestFile = -1 Then
        BinResError = 21000 + DestOpenInfo.nErrCode
        BinResStatus = Ident & "Error opening destination file"
        LZClose hfSrcFile
        Exit Sub
    End If

    lDestLen = LZCopy(hfSrcFile, hfDestFile)
    If lDestLen < 0 Then
        BinResError = 22000 + lDestLen
        BinResStatus = Ident & "Error expanding file"
        LZClose hfSrcFile
        LZClose hfDestFile
        Exit Sub
    End If

    LZClose hfSrcFile
    LZClose hfDestFile

    BinResError = 0
    BinResStatus = Ident & "Expanded OK to " & lDestLen & " byte file"

ExpandExit:

Exit Sub

ExpandError:

    BinResError = Err
    BinResStatus = Ident & "Error " & Err & ": " & Error

Resume ExpandExit

End Sub

Function BinResGet(BSId As String) As String

Const Ident = "BinResGet: "
Dim ChPos As Integer, EntIdx As Integer
Dim DatOff As Long, DatLen As Long
Dim DatBuff As String

    ChPos = InStr(BR_LookupId, BR_LOOKUPBEGID & BSId & BR_LOOKUPENDID)
    If ChPos = 0 Then
        BinResError = 32001
        BinResStatus = Ident & "Resource identifier not found"
        BinResGet = ""
        Exit Function
    End If
        
    EntIdx = BinResStrInt(Mid(BR_LookupId, ChPos - 2, 2))
    DatLen = BinResStrLong(Mid(BR_IdxEntries, EntIdx + 4, 4))
    If DatLen < 0 Or DatLen > 32767 Then
        BinResError = 32003
        BinResStatus = Ident & "Resource data length invalid for get string = " & DatLen
        BinResGet = ""
        Exit Function
    End If
    
    If DatLen = 0 Then
        BinResGet = ""
    Else
        DatOff = BinResStrLong(Mid(BR_IdxEntries, EntIdx, 4))
        DatBuff = Space(DatLen)
        Get #BR_Fid, DatOff, DatBuff
        BinResGet = DatBuff
    End If

    BinResError = 0
    BinResStatus = ""

End Function

Sub BinResGetFile(BSId As String, BGFile As String)

Const Ident = "BinResGetFile: "
Dim ChPos As Integer, EntIdx As Integer, FFid As Integer
Dim DatOff As Long, DatLen As Long, BOff As Long
Dim BData As String

    On Error GoTo GetFileError

    ChPos = InStr(BR_LookupId, BR_LOOKUPBEGID & BSId & BR_LOOKUPENDID)
    If ChPos = 0 Then
        BinResError = 32001
        BinResStatus = Ident & "Resource identifier not found"
        Exit Sub
    End If
        
    EntIdx = BinResStrInt(Mid(BR_LookupId, ChPos - 2, 2))
    DatLen = BinResStrLong(Mid(BR_IdxEntries, EntIdx + 4, 4))
    If DatLen < 0 Or DatLen > BR_MAXDATALEN Then
        BinResError = 32004
        BinResStatus = Ident & "Resource data length invalid for get file = " & DatLen
        Exit Sub
    End If
    
    On Error Resume Next
    Kill BGFile
    On Error GoTo GetFileError

    FFid = FreeFile
    Open BGFile For Binary Access Read Write Lock Read Write As #FFid
    If DatLen > 0 Then
        BOff = 1
        DatOff = BinResStrLong(Mid(BR_IdxEntries, EntIdx, 4))
        BData = Space(BR_BUFFSIZE)
        Do Until BOff > DatLen
            If DatLen - BOff < BR_BUFFSIZE Then BData = Space(DatLen - BOff + 1)
            Get #BR_Fid, DatOff, BData
            Put #FFid, BOff, BData
            DatOff = DatOff + BR_BUFFSIZE
            BOff = BOff + BR_BUFFSIZE
        Loop
    End If
    Close #FFid

    BinResError = 0
    BinResStatus = ""

GetFileExit:

Exit Sub

GetFileError:

    BinResError = Err
    BinResStatus = Ident & "Error " & Err & ": " & Error

Resume GetFileExit

End Sub

Function BinResGetIds() As String

Dim PrevPos As Integer, ChPos As Integer
Dim IdsList As String

    IdsList = ""
    PrevPos = 0
    Do
        ChPos = InStr(PrevPos + 3, BR_LookupId, BR_LOOKUPBEGID)
        If ChPos = 0 Then Exit Do
        PrevPos = InStr(ChPos + 1, BR_LookupId, BR_LOOKUPENDID)
        IdsList = IdsList & Mid(BR_LookupId, ChPos + 1, PrevPos - ChPos - 1) & vbNullChar
    Loop
    
    BinResGetIds = IdsList
    BinResError = 0
    BinResStatus = ""

End Function

Function BinResGetLength(BSId As String) As Long

Const Ident = "BinResGetLength: "
Dim ChPos As Integer, EntIdx As Integer

    ChPos = InStr(BR_LookupId, BR_LOOKUPBEGID & BSId & BR_LOOKUPENDID)
    If ChPos = 0 Then
        BinResError = 32001
        BinResStatus = Ident & "Resource identifier not found"
        BinResGetLength = -1
    Else
        EntIdx = BinResStrInt(Mid(BR_LookupId, ChPos - 2, 2))
        BinResGetLength = BinResStrLong(Mid(BR_IdxEntries, EntIdx + 4, 4))
    End If

End Function

Function BinResIntStr(ByVal IntVal As Integer) As String

    BinResIntStr = Chr(IntVal Mod 256) + Chr(IntVal \ 256)

End Function

Function BinResLongStr(ByVal LongVal As Long) As String

Dim HexVal As String, StrVal As String * 4, Idx As Integer

    HexVal = Right("0000000" & Hex(LongVal), 8)
    For Idx = 1 To 4
        Mid(StrVal, Idx, 1) = Chr(CInt("&H" & Mid(HexVal, 9 - Idx * 2, 2)))
    Next Idx
    BinResLongStr = StrVal

End Function

Sub BinResOpen(ByVal BSFilePath As String, ByVal BSInitIndexLength As Integer)

Const Ident = "BinResOpen: "
Dim EntIdx As Integer, ResIdLen As Integer
Dim EmptyIdxEnts As String
    
    On Error GoTo OpenError

    If BR_Fid Then
        BinResError = 32002
        BinResStatus = "Already open"
        Exit Sub
    End If

    BR_Fid = FreeFile
    BR_Base = -99999999
    BR_IdxEntries = ""
    BR_IdxUpdated = False
    BR_Free = -99999999
    BR_LookupId = ""
    
    Select Case BSInitIndexLength
      
      Case 0
        ' Open for read only
        Open BSFilePath For Binary Access Read As #BR_Fid
        ' Get offset to index block
        Get #BR_Fid, BR_BaseOff, BR_Base
        If BR_Base < BR_MINBASE Then
            BinResError = 32001
            BinResStatus = Ident & "Resource is empty"
            Close #BR_Fid
            BR_Fid = 0
            Exit Sub
        End If
        ' Get max length of index entries
        Get #BR_Fid, BR_Base, BR_IdxMaxEntsLen
        ' Get used length of index entries
        Get #BR_Fid, BR_Base + 2, BR_IdxUsedEntsLen
        ' Get index block entries
        BR_IdxEntries = Space(BR_IdxUsedEntsLen)
        Get #BR_Fid, BR_Base + 4, BR_IdxEntries
        ' Build the identifier lookup string
        BR_LookupId = BR_LOOKUPBEGID
        EntIdx = 1  ' Subscript of first index entry
        Do Until EntIdx > BR_IdxUsedEntsLen
            ResIdLen = Asc(Mid(BR_IdxEntries, EntIdx + 8, 1))
            BR_LookupId = BR_LookupId & BinResIntStr(EntIdx) & BR_LOOKUPBEGID & Mid(BR_IdxEntries, EntIdx + 9, ResIdLen) & BR_LOOKUPENDID
            EntIdx = EntIdx + 9 + ResIdLen
        Loop
    
    Case Is < 64
        BinResError = 32005
        BinResStatus = Ident & "Invalid initial index length"
        
    Case Else
        ' Open for write
        Open BSFilePath For Binary Access Read Write As #BR_Fid
        ' Get offset to index block
        Get #BR_Fid, BR_BaseOff, BR_Base
        If BR_Base < BR_MINBASE Then
            ' No existing index, calculate base
            BR_Base = (LOF(BR_Fid) Or &HF) + 2
            If BR_Base < BR_MINBASE Then BR_Base = BR_MINBASE
            Put #BR_Fid, BR_BaseOff, BR_Base
        End If
        ' Initialise index block to empty
        BR_IdxMaxEntsLen = BSInitIndexLength
        BR_Free = ((BR_Base + 4 + BR_IdxMaxEntsLen) Or &HF) + 2
        BR_IdxUsedEntsLen = 0
        EmptyIdxEnts = String(BR_Free - BR_Base - 4, 255)
        BR_IdxEntries = ""
        BR_LookupId = ""
        ' Update the resource file
        Put #BR_Fid, BR_Base, BR_IdxMaxEntsLen
        Put #BR_Fid, BR_Base + 2, BR_IdxUsedEntsLen
        Put #BR_Fid, BR_Base + 4, EmptyIdxEnts
    End Select

    BinResError = 0
    BinResStatus = ""

OpenExit:

Exit Sub

OpenError:

    BinResError = Err
    BinResStatus = Ident & "Error " & Err & ": " & Error
    Close #BR_Fid
    BR_Fid = 0

Resume OpenExit
    
End Sub

Function BinResStrInt(ByVal StrVal As String) As Integer

    BinResStrInt = Asc(StrVal) + Asc(Mid(StrVal, 2, 1)) * 256

End Function

Function BinResStrLong(ByVal StrVal As String) As Long

    BinResStrLong = Asc(StrVal) + Asc(Mid(StrVal, 2, 1)) * 256& + Asc(Mid(StrVal, 3, 1)) * 65536 + Asc(Mid(StrVal, 4, 1)) * 16777216

End Function

